%----------------------------------------------------
%
% Main program running usfos analysis in paralel
%
%   Marius Karlsson NTNU 2017
%
%---------------------------------------------------


clear all
close all


%Defining cross sectional marameter for braces and chords of interest
%properties   Area        ,Iy         ,Iz         ,z
geom_287=     [2.32469E-01,6.99900E-03,8.78900E-03,0.6629];
geom_288=geom_287;
%Geometrical properties
ro450=0.324/2;
t450=0.028;
geom_450=     [pi*(ro450^2-(ro450-t450)^2),0.25*pi*(ro450^4-(ro450-t450)^4),0.25*pi*(ro450^4-(ro450-t450)^4),2*ro450];
geom_451=geom_450;
geom_170=geom_450;
geom_169=geom_450;
SCF_AC=3.2155
SCF_AS=3.2283
SCF_MIP=1.9032
SCF_MOP=1.8839


tmax=3*3600;

%Dummy
type={'Spectrum'}
CM_vec=[2.0,1.8];




%Selecting type of wave (dummy)
typ=1



%Gathering information about number of cores in computer
n_cores=feature('numCores')

%Creating folders for every worker (core)
for i=1:n_cores
    mkdir(['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/head_in_timesim.fem'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/modify_input.m'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/model.fem'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/dynres_input.txt'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/usfos.txt'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/SCF_stress.m'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/generate_jonswap_file.m'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/List_drag_coefficients.m'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/verticals_0.txt'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/verticals_30.txt'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/elements.txt'],['H:/timesim/timesimDir' num2str(i)])
    copyfile(['H:/timesim/nodes.txt'],['H:/timesim/timesimDir' num2str(i)])
end


dir_vec=linspace(0,180,2*n_cores+1)
disp_x={}
disp_x={}
disp_y={}
disp_z={}
overturn={}
base_shear={}
surf_elev={}




%Number of integration points
NP=[2,3,4,5,6,7,8,9,10,11,12,13,14,15,2,3,4,5,6,7,8,9,10,11,12,13,14,15]
NP_deep=[2,3,4,5,6,7,8,9,10,11,12,13,14,15,4,4,4,4,4,4,4,4,4,4,4,4,4,4]



%Creating a vector of HS and TP in whiche every member are run in analysis
load('TP_ekofisk.mat')
load('scatter_matrix_ekofisk.mat')
load('HS_ekofisk.mat')
TP_vec=0
HS_vec=0
for I=1:size(scatter_matrix_ekofisk,1)
    for J=1:size(scatter_matrix_ekofisk,2)
        if scatter_matrix_ekofisk(I,J)~=0&&HS_ekofisk(I)>0
            TP_vec(end+1)=TP_ekofisk(J);
            HS_vec(end+1)=HS_ekofisk(I);
        end
    end
end
TP_vec(1)=[];
HS_vec(1)=[]



% Drag coefficients
CD_vec=[1.15,1.15]

%static vs dynamic and damping
rel_vel={'rel_velo','rel_velo','rel_velo','rel_velo','rel_velo',      '','','','','',     '','','','',''}
static={'dynamic','dynamic','dynamic','dynamic','dynamic',     'static','static','static','static','static',     'dynamic','dynamic','dynamic','dynamic','dynamic'}

%Frequency components
nFreq=5000;

%Time increment
dt=[1,2,4,8,16,32,64]*0.025
dtJ=4
k=1
parfor j=1:length(TP_vec)
    
    %Assigning varable to worker ID
    t = getCurrentTask()
    
    %Changing to worker temp directory
    cd(['H:/timesim/timesimDir' num2str(t.ID)]);
    
    
    
    %Generating stochastic input file
    generate_jonswap_file(HS_vec(j),TP_vec(j),seed_vec(j),1,25,nFreq,dir_vec(1))
    
    %Replacing parameters of input file
    modify_input(HS_vec(j),TP_vec(j),dir_vec(1),tmax,dt(dtJ),type{typ},CD_vec(1),CM_vec(typ),'head_in_timesim.fem',NP(19),NP_deep(17),0,nFreq,'rel_velo','static');
    
    %Creating special coeficients for chords
    if CD_vec(1)==1.15
        List_drag_coefficients('head.fem')
    end
    
    %Running USFOS and dynres
    system([char(34) 'H:\USFOS (64 bit) 8.8\bin\usfos.exe' char(34) '<' char(34) 'H:\timesim\timesimDir' num2str(t.ID) '\usfos.txt' char(34)])
    system([char(34) 'H:\USFOS (64 bit) 8.8\bin\dynres.exe' char(34) '<' char(34) 'H:\timesim\timesimDir' num2str(t.ID) '\dynres_input.txt' char(34)])
    
    
    axial_x_450{j,k}=dlmread('result_Element_Force_Elem_450_End_1_Dof_1.plo')
    bending_y_450{j,k}=dlmread('result_Element_Force_Elem_450_End_1_Dof_5.plo')
    bending_z_450{j,k}=dlmread('result_Element_Force_Elem_450_End_1_Dof_6.plo')
    
    axial_x_451{j,k}=dlmread('result_Element_Force_Elem_451_End_1_Dof_1.plo')
    bending_y_451{j,k}=dlmread('result_Element_Force_Elem_451_End_1_Dof_5.plo')
    bending_z_451{j,k}=dlmread('result_Element_Force_Elem_451_End_1_Dof_6.plo')
    
    surf_elev{j,k}=dlmread('result_Surface_Elevation.plo')
    
    
    A_450=1
    Dy450=1
    Iy_450=1
    Dz450=1
    Iz_450=1
    
    
    %Stress components (bending axial etc.)
    Sx450{j,k}=axial_x_450{j,k}(:,2)/geom_450(1);
    Smy450{j,k}=bending_y_450{j,k}(:,2)*0.5*geom_450(4)/geom_450(2);
    Smz450{j,k}=bending_z_450{j,k}(:,2)*0.5*geom_450(4)/geom_450(3);
    
    Sx451{j,k}=axial_x_451{j,k}(:,2)/geom_450(1);
    Smy451{j,k}=bending_y_451{j,k}(:,2)*0.5*geom_450(4)/geom_450(2);
    Smz451{j,k}=bending_z_451{j,k}(:,2)*0.5*geom_450(4)/geom_450(3);
    
    
    %Hot spot stress
    S_vec_450{j,k}=SCF_stress(Sx450{j,k},Smy450{j,k},Smz450{j,k},SCF_AS,SCF_AC,SCF_MIP,SCF_MOP)
    
    S_vec_451{j,k}=SCF_stress(Sx451{j,k},Smy451{j,k},Smz451{j,k},SCF_AS,SCF_AC,SCF_MIP,SCF_MOP)
    
    % Performing rainflow counting
    
    %         N_rfm=100
    %         [RFM{j,k},u{j,k},param{j,k}] = dat2rfm(S_vec_450{j,k}{1},0,N_rfm)
    %
    %         RFM_rec{j,k}=0
    %         for i_rfm=1:N_rfm
    %             for j_rfm=1:N_rfm
    %                 if RFM{j,k}(i_rfm,j_rfm)~=0
    %                 for k_rfm=1:RFM{j,k}(i_rfm,j_rfm)
    %                 RFM_rec{j,k}(end+1)=u{j,k}(j_rfm)-u{j,k}(i_rfm);
    %                 end
    %                 end
    %             end
    %         end
    %
    
end

%Saving result files
cd(['H:/timesim/'])
save('disp_428_x.mat','disp_x')
save('Sx450.mat','Sx450')
save('Smy450.mat','Smy450')
save('Smz450.mat','Smz450')


save('axial_x_450.mat','axial_x_450')
save('bending_y_450.mat','bending_y_450')
save('bending_z_450.mat','bending_z_450')



save('disp_428_y.mat','disp_y')
save('disp_428_z.mat','disp_z')
save('overturn.mat','overturn')
save('surf_elev.mat','surf_elev')
save('base_shear.mat','base_shear')
save('S_vec_450.mat','S_vec_450')
%save('RFM_rec.mat','RFM_rec')